package main

import (
	"flag"
	"golang.org/x/net/context"
	"google.golang.org/grpc"
	"loafle.com/overflow/crawler_go"
	pb "loafle.com/overflow/crawler_go/grpc"
	"log"
	"net"
	"loafle.com/overflow/redis_protocol_crawler_go"
	"loafle.com/overflow/activedirectory_protocol_crawler_go"
	"loafle.com/overflow/dns_protocol_crawler_go"
	"loafle.com/overflow/ftp_protocol_crawler_go"
	"loafle.com/overflow/ftps_protocol_crawler_go"
	"loafle.com/overflow/imap_protocol_crawler_go"
	"loafle.com/overflow/ldap_protocol_crawler_go"
	"loafle.com/overflow/mongodb_protocol_crawler_go"
	"loafle.com/overflow/mssql_protocol_crawler_go"
	"loafle.com/overflow/mysql_protocol_crawler_go"
	"loafle.com/overflow/pgsql_protocol_crawler_go"
	"loafle.com/overflow/netbios_protocol_crawler_go"
	"loafle.com/overflow/oracle_protocol_crawler_go"
	"loafle.com/overflow/pop3_protocol_crawler_go"
	"loafle.com/overflow/rmi_protocol_crawler_go"
	"loafle.com/overflow/smb_protocol_crawler_go"
	"loafle.com/overflow/smtp_protocol_crawler_go"
	"loafle.com/overflow/snmpv2c_protocol_crawler_go"
	"loafle.com/overflow/snmpv3_protocol_crawler_go"
	"loafle.com/overflow/ssh_protocol_crawler_go"
	"loafle.com/overflow/telnet_protocol_crawler_go"
	"loafle.com/overflow/wmi_protocol_crawler_go"
	"loafle.com/overflow/cassandra_protocol_crawler_go"
	"loafle.com/overflow/http_protocol_crawler_go"
)

var g_crawlers map[string]crawler.Crawler

func AddDelegate(name string, c crawler.Crawler) {
	if g_crawlers == nil {
		g_crawlers = make(map[string]crawler.Crawler, 0)
	}
	g_crawlers[name] = c
}

type ConfigServer struct {
}

func (s *ConfigServer) Add(c context.Context, in *pb.Input) (*pb.Output, error) {
	return nil, nil
}
func (s *ConfigServer) Remove(c context.Context, in *pb.Input) (*pb.Output, error) {
	return nil, nil
}
func (s *ConfigServer) Init(c context.Context, in *pb.InputArray) (*pb.Output, error) {
	return nil, nil
}

type DataServer struct {
}

func (s *DataServer) Get(c context.Context, in *pb.Input) (*pb.Output, error) {

	output := &pb.Output{}

	if c, ok := g_crawlers[in.Name.String()]; ok {
		rd,err := c.Get(in.Id)
		if err != nil {
			// process error
		}
		output.Data = rd
	} else {
		output.Data = []byte("Not Assign Crawler")
	}
	return output, nil
}


func initCrawlers() {
	g_crawlers = make(map[string]crawler.Crawler,0)
	
	AddDelegate(pb.Crawlers_HEALTH_ACTIVEDIRECTORY.String(), activedirectory_protocol_crawler_go.NewActiveDirectoryHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_DNS.String(), dns_protocol_crawler_go.NewDNSHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_FTP.String(), ftp_protocol_crawler_go.NewFTPHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_FTPS.String(), ftps_protocol_crawler_go.NewFTPSHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_IMAP.String(), imap_protocol_crawler_go.NewIMAPHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_LDAP.String(), ldap_protocol_crawler_go.NewLDAPHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_MONGODB.String(), mongodb_protocol_crawler_go.NewMongoDBHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_MSSQL.String(), mssql_protocol_crawler_go.NewMSSqlHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_MYSQL.String(), mysql_protocol_crawler_go.NewMysqlHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_MARIADB.String(), mysql_protocol_crawler_go.NewMysqlHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_PGSQL.String(), pgsql_protocol_crawler_go.NewPGSqlHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_NETBIOS.String(), netbios_protocol_crawler_go.NewNetbiosHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_ORACLE.String(), oracle_protocol_crawler_go.NewOracleHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_POP3.String(), pop3_protocol_crawler_go.NewPOP3HeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_REDIS.String(), redis_protocol_crawler_go.NewRedisHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_RMI.String(), rmi_protocol_crawler_go.NewRMIHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_SMB.String(), smb_protocol_crawler_go.NewSMBHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_SMTP.String(), smtp_protocol_crawler_go.NewSMTPHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_SNMPV2C.String(), snmpv2c_protocol_crawler_go.NewSNMPV2CHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_SNMPV3.String(), snmpv3_protocol_crawler_go.NewSNMPV3HeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_SSH.String(), ssh_protocol_crawler_go.NewSSHHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_TELNET.String(), telnet_protocol_crawler_go.NewTelnetHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_WMI.String(), wmi_protocol_crawler_go.NewWMIHeahthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_CASSANDRA.String(), cassandra_protocol_crawler_go.NewCassandraHealthCrawler())
	AddDelegate(pb.Crawlers_HEALTH_HTTP.String(), http_protocol_crawler_go.NewHTTPHealthCrawler())
}

func start(rc pb.ConfigServer,rd pb.DataServer) {

	port := flag.String("Port", "50000", "RPC Port Default 50000")
	flag.Parse()
	address := "localhost:" + *port

	log.Println("address : " + address)
	lis, err := net.Listen("tcp", address)
	if err != nil {
		log.Fatalf("failed to listen: %v", err)
	}
	s := grpc.NewServer()
	pb.RegisterConfigServer(s,rc)
	pb.RegisterDataServer(s,rd)
	if err := s.Serve(lis); err != nil {
		log.Fatalf("failed to serve: %v", err)
	}
}

func main() {
	initCrawlers()
	start(&ConfigServer{},&DataServer{})
}
